*** command.c.orig	Thu Dec  1 11:13:42 1994
--- command.c	Tue Dec 13 10:19:22 1994
***************
*** 41,46 ****
--- 41,53 ----
   * 19 September 1992  Lawrence Crowl  (crowl@cs.orst.edu)
   * Added user-specified bases for log scaling.
   * 
+  * 19 June 1993 Kevin Rodgers (rodgers@lvs-emh.lvs.loral.com) Extended the
+  * mods made by Jack Veenstra to allow passing the x values of a plot data
+  * set through a filter; the keyword is xthru.  An xthru keyword must come
+  * before a thru keyword.  Also added a ythru keyword to make the interface
+  * consistent.  Also added capability to read Fortran-style unformatted file
+  * (works with all Unixes I know; must be modified for VMS and maybe PCs)
+  *
   * There is a mailing list for gnuplot users. Note, however, that the
   * newsgroup 
   *	comp.graphics.gnuplot 
***************
*** 382,387 ****
--- 389,397 ----
  /* jev -- for passing data thru user-defined function */
  static struct udft_entry ydata_func;
  
+ /* kmr -- for passing x data thru function */
+ static struct udft_entry xdata_func;
+ 
  /* support for replot command */
  char            replot_line[MAX_LINE_LEN + 1] = "";
  static int      plot_token;	/* start of 'plot' command */
***************
*** 948,953 ****
--- 958,972 ----
  	double val[5], v[5];
  	float fval[5];	/* for use in sscanf */
  
+ 	/* kmr -- modification to allow read of binary files */
+ 	TBOOLEAN file_is_binary = FALSE;
+ 	TBOOLEAN bin8 = FALSE;
+ 	TBOOLEAN EOF_Flag = FALSE;
+ 	double ddata[5];
+ 	float  fdata[5];
+ 	int recLen, bytes_per_word, nSeek, k;
+ 	long int initial_seek, seek[5];
+ 
  	/* close forgotten input file (in case of a syntax error) */
  	if( data_fp ) {
  #if defined(unix) || defined(PIPES)
***************
*** 976,981 ****
--- 995,1017 ----
  	if ((data_fp = fopen(data_file, "r")) == (FILE *) NULL)
  		os_error("can't open data file", c_token);
  
+ 	/* kmr -- modifications for handling binary files */
+ 	if (is_binary_file(data_fp)) {
+ 	  file_is_binary = TRUE;
+ 
+ 	  /* kmr -- I know too little about pipes to allow binary read from one */
+ #ifdef PIPES
+ 	  if( pipe_open ) {
+ 	    pclose(data_fp);
+ 	    data_fp=NULL;
+ 	    pipe_open=FALSE;
+ 	    int_error("binary data from pipes is not implemented", NO_CARET);
+ 	  }
+ #endif /* PIPES */
+ 	  data_fp = freopen(data_file, "rb", data_fp);
+ 	}
+ 	rewind(data_fp);
+ 
  	format[0] = '\0';
  	for (i=0; i<5; i++)
  		fcol[i] = i+1;
***************
*** 983,990 ****
  	using = 0;
  	c_token++;			/* skip data file name */
  
  	/* jev -- support for passing data from file thru user function */
! 	if (almost_equals(c_token, "thru$")) {
  		c_token++;
  		if (ydata_func.at)
  			free(ydata_func.at);
--- 1019,1053 ----
  	using = 0;
  	c_token++;			/* skip data file name */
  
+ 	if (equals(c_token, "bin8")) {
+ 	  c_token++;
+ 	  bin8 = TRUE;
+ 	}
+         
+ 	if (equals(c_token, "bin4")) {
+ 	  c_token++;
+ 	  bin8 = FALSE;
+ 	}
+         
  	/* jev -- support for passing data from file thru user function */
! 	/* kmr -- extend to xdata filtering with "xthru" keyword */
! 	/* kmr -- here's the x axis version */
! 	if (equals(c_token, "xthru")) {
! 		c_token++;
! 		if (xdata_func.at)
! 			free(xdata_func.at);
! 		dummy_func = &xdata_func;
! 		xdata_func.at = perm_at();
! 	} else {
! 		if (xdata_func.at)
! 			free(xdata_func.at);
! 		xdata_func.at = NULL;
! 	}
! 
! 	/* kmr -- modified jev's version to accept either "thru" or "ythru"
! 	 *        (ythru adds consistency with xthru)
! 	 */
! 	if (equals(c_token, "thru") || equals(c_token, "ythru")) {
  		c_token++;
  		if (ydata_func.at)
  			free(ydata_func.at);
***************
*** 1004,1014 ****
  			struct value a;
  			for (i=0; i<5; i++)
  				fcol[i] = -1;
! 			fcol[0] = (int)magnitude(const_express(&a));
  			for (i=1; equals(c_token,":") && i<5; i++) {
  				c_token++;      /* skip ":" */
  				fcol[i] = (int)magnitude(const_express(&a));
  			}
  		}
  
  		if (!END_OF_COMMAND && isstring(c_token)) {
--- 1067,1081 ----
  			struct value a;
  			for (i=0; i<5; i++)
  				fcol[i] = -1;
! 			/* fix a bug in gnuplot 3.5 (was ok in 3.4!)
! 			   so one number is Y column with X column equal 1 */
! 			fcol[0] = fcol[1] = (int)magnitude(const_express(&a));
  			for (i=1; equals(c_token,":") && i<5; i++) {
  				c_token++;      /* skip ":" */
  				fcol[i] = (int)magnitude(const_express(&a));
  			}
+ 			if (i==1)  /* only y column given */
+ 			  fcol[0] = 1;
  		}
  
  		if (!END_OF_COMMAND && isstring(c_token)) {
***************
*** 1049,1055 ****
  	l_num = 0;
  	datum = 0;
  	i = 0;
! 	while (fgets(line, MAX_LINE_LEN, data_fp) != (char *) NULL) {
  		l_num++;
  		if (is_comment(line[0]))
  	    	continue;		/* ignore comments */
--- 1116,1192 ----
  	l_num = 0;
  	datum = 0;
  	i = 0;
! 
! 	/* kmr -- set up binary file parameters */
! 	if (file_is_binary) {
! 	  rewind(data_fp);
! 	  fread(&recLen, sizeof(int), 1, data_fp);  /* first int is record length */
! 	  rewind(data_fp);
! 	  /*  set bytes per word; default is 4 (single-precision) */
! 	  if (bin8)
! 	    bytes_per_word = 8;
! 	  else
! 	    bytes_per_word = 4;
! 
! 	  /* get number of valid columns */
! 	  for (nSeek=0; nSeek<5 && ncol[nSeek]>=0 && ncol[nSeek]-1 < (recLen/bytes_per_word) ; nSeek++)
! 	    ;  /* after loop is exited, nSeek is number of columns */
! 	  if (nSeek == 0)
! 	    int_error("no valid data columns specified!", NO_CARET);  /* no valid columns!! */
! 	  /*
! 	   * instead of reading a whole line, we will seek to the specific record locations
! 	   * tests on DECstations indicate this is faster than reading whole record.
! 	   * expected record format is: (4 byte int) (data records) (4 byte int) which
! 	   * is straight out of DEC RISC Fortran manual; seems to be a de facto Unix std.
! 	   */
! 	  initial_seek = 4 + bytes_per_word*(scol[0] - 1);
! 	  for (j=0; j < nSeek-1; j++)
! 	    seek[j] = (scol[j+1] - scol[j] - 1)*bytes_per_word;
! 	  seek[nSeek-1] = (recLen - scol[nSeek-1]*bytes_per_word + 4) + initial_seek;
! 	  fseek(data_fp, initial_seek, SEEK_CUR);  /* position file at beginning of first data element */
! 	  if (feof(data_fp) || ferror(data_fp))
! 	    os_error("error or eof on data file",c_token);
! 	}
! 
! 
! 	/* while (fgets(line, MAX_LINE_LEN, data_fp) != (char *) NULL) { */
! 	for ( ; ; ) {
! 	  if (file_is_binary) {
! 	    for (j=0; j<nSeek; j++) {
! 	      if (bin8) 
! 		fread(&val[j], sizeof(double), 1, data_fp);
! 	      else
! 		fread(&fdata[j], sizeof(float), 1, data_fp);
! 	      if (feof(data_fp) || ferror(data_fp)) {
! 		EOF_Flag = TRUE; 
! 		break;
! 	      }
! 	      fseek(data_fp, seek[j], SEEK_CUR);
! 	    }
! 	    if (!EOF_Flag) {
! 	      l_num++;
! 	      if (i >= this_plot->p_max)
! 		/*
! 		 * overflow about to occur. Extend size of points[] array. We
! 		 * either double the size, or add 1000 points, whichever is a
! 		 * smaller increment. Note i=p_max.
! 		 */
! 		cp_extend(this_plot, i + (i < 1000 ? i : 1000));
! 	      /* convert to double if necessary */
! 	      if (!bin8)
! 		for(j=0; j<nSeek; j++)
! 		  val[j] = (double) fdata[j];
! 
! 	      /* stuff into correct slots in v array */
! 	      for (j=0; j<5 && ncol[j]>=0 && ncol[j]-1<nSeek ; j++)
! 		if (ncol[j])
! 		  v[j] = val[ncol[j]-1];
! 		else
! 		  v[j] = datum;	/* using 0:n */
! 	    }
! 	  }
! 	  else {
! 	    if (fgets(line, MAX_LINE_LEN, data_fp) != (char *) NULL) {
  		l_num++;
  		if (is_comment(line[0]))
  	    	continue;		/* ignore comments */
***************
*** 1116,1122 ****
  				else
  					v[j] = datum;	/* using 0:n */
  		}
! 
  		switch (j) {
  		    case 1: {		/* only one number */
  		    	/* x is index, assign number to y */
--- 1253,1263 ----
  				else
  					v[j] = datum;	/* using 0:n */
  		}
! 	      }
! 	      else
! 		EOF_Flag = TRUE;
! 	  }
! 	  if (!EOF_Flag) {
  		switch (j) {
  		    case 1: {		/* only one number */
  		    	/* x is index, assign number to y */
***************
*** 1160,1165 ****
--- 1301,1309 ----
  				int_error(line, c_token);
  		    }
  		}
+ 	      }
+ 	    else
+ 	      break;
  	}
  	this_plot->p_count = i;
  	cp_extend(this_plot, i);	/* shrink to fit */
***************
*** 1184,1189 ****
--- 1328,1334 ----
      double          width;
  {
      struct coordinate GPHUGE *cp = &(this_plot->points[i]);
+     double xtmp;
  
      /* the easy part: */
      cp->type = INRANGE;
***************
*** 1194,1199 ****
--- 1339,1354 ----
      cp->z = width;
  
      /* jev -- pass data values thru user-defined function */
+     /* kmr -- added support for x values */
+     if (xdata_func.at) {
+         struct value    val;
+ 
+ 	(void) Gcomplex(&xdata_func.dummy_values[0], x, 0.0);
+ 	evaluate_at(xdata_func.at, &val);
+ 	cp->x = real(&val);
+     }
+     xtmp = cp->x;   /* kmr - save x after filtering but before log transform */
+ 
      if (ydata_func.at) {
  	struct value    val;
  
***************
*** 1225,1237 ****
       * The yrange is handled later, once we know whether to include ylow,
       * yhigh in the calculation. See adjust_yrange()
       */
      if (cp->type == INRANGE)
! 	if (autoscale_lx || inrange(x, xmin, xmax)) {
  	    if (autoscale_lx) {
! 		if (x < xmin)
! 		    xmin = x;
! 		if (x > xmax)
! 		    xmax = x;
  	    }
  	} else {
  	    cp->type = OUTRANGE;
--- 1380,1395 ----
       * The yrange is handled later, once we know whether to include ylow,
       * yhigh in the calculation. See adjust_yrange()
       */
+     /* kmr -- changed x to xtmp since x may have been passed through
+      *        a function but don't want log value of x
+      */
      if (cp->type == INRANGE)
! 	if (autoscale_lx || inrange(xtmp, xmin, xmax)) {
  	    if (autoscale_lx) {
! 		if (xtmp < xmin)
! 		    xmin = xtmp;
! 		if (xtmp > xmax)
! 		    xmax = xtmp;
  	    }
  	} else {
  	    cp->type = OUTRANGE;
***************
*** 2245,2255 ****
  		/* ignore this now */
  		c_token++;
  
  		/*
  		 * jev -- support for passing data from file thru user
  		 * function
  		 */
! 		if (almost_equals(c_token, "thru$")) {
  		    struct udft_entry tmp;
  		    c_token++;
  		    dummy_func = &tmp;
--- 2403,2427 ----
  		/* ignore this now */
  		c_token++;
  
+ 		/* support for bin4 or bin8 keyword */
+ 		if (equals(c_token, "bin8") || equals(c_token, "bin4"))
+ 		  c_token++;   /* skip */
+ 
  		/*
  		 * jev -- support for passing data from file thru user
  		 * function
  		 */
! 		/*
! 		 * kmr -- added support for x axis thru user function
! 		 * and made ythru keyword equivalent to thru
! 		 */
! 		if (equals(c_token, "xthru")) {
! 		    struct udft_entry tmp;
! 		    c_token++;
! 		    dummy_func = &tmp;
! 		    (void) temp_at();
! 		}
! 		if (equals(c_token, "thru") || equals(c_token, "ythru")) {
  		    struct udft_entry tmp;
  		    c_token++;
  		    dummy_func = &tmp;
***************
*** 2433,2440 ****
  	/* Now actually fix the plot pairs to be single plots. */
  	parametric_fixup(first_plot, &plot_num, &x_min, &x_max);
      }
!     if (strcmp(term_tbl[term].name, "table") == 0)
! 	print_table();
      else
  	do_plot(first_plot, plot_num, x_min, x_max, y_min, y_max);
      cp_free(first_plot);
--- 2605,2613 ----
  	/* Now actually fix the plot pairs to be single plots. */
  	parametric_fixup(first_plot, &plot_num, &x_min, &x_max);
      }
!     if (strcmp(term_tbl[term].name, "table") == 0) {
!       print_table();
!     }
      else
  	do_plot(first_plot, plot_num, x_min, x_max, y_min, y_max);
      cp_free(first_plot);
***************
*** 2801,2806 ****
--- 2974,2983 ----
  	    if (isstring(c_token)) {	/* data file to plot */
  		/* ignore this now */
  		c_token++;
+ 
+ 		if (equals(c_token, "bin8") || equals(c_token, "bin4"))
+ 		  c_token++;   /* skip bin8 or bin4 */
+ 
  		if (almost_equals(c_token, "i$ndex")) {
  		    struct value a;
  		    int index;
